@@ -7,53 +7,36 @@ using namespace IMG;
 
 enum Charset {Normal, Symbol, Special};
 
-static quint8 NORMAL_CHARS[] = {
-	' ', 'A', 'B', 'C', 'D', 'E', 'F', 'G',
-	'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
-	'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W',
-	'X', 'Y', 'Z',  '~', '~', '~', ' ', ' ',
-	'0', '1', '2', '3', '4', '5', '6', '7',
-	'8', '9', '~', '~', '~', '~', '~', '~'
-};
-
-static quint8 SYMBOL_CHARS[] = {
-	'@', '!', '"', '#', '$', '%', '&', '\'',
-	'(', ')', '*', '+', ',', '-', '.', '/',
-	'~', '~', '~', '~', '~', '~', '~', '~',
-	'~', '~', ':', ';', '<', '=', '>', '?',
-	'~', '~', '~', '~', '~', '~', '~', '~',
-	'~', '~', '~', '[', '\\', ']', '^', '_'
-};
-
-static quint8 SPECIAL_CHARS[] = {
-	'`', 'a', 'b', 'c', 'd', 'e', 'f', 'g',
-	'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
-	'p', 'q', 'r', 's', 't', 'u', 'v', 'w',
-	'x', 'y', 'z', '~', '~', '~', '~', '~',
-	'0', '1', '2', '3', '4', '5', '6', '7',
-	'8', '9', '~', '~', '~', '~', '~', '~'
-};
-
 static bool isAllUpperCase(const QString &str)
 {
 	if (str.isEmpty())
 		return false;
-	for (int i = 0; i < str.size(); i++)
-		if (str.at(i).isLetter() && !(str.at(i).isUpper()
-		  || str.at(i) == QChar(0x00DF)))
+
+	for (int i = 0; i < str.size(); i++) {
+		QChar c(str.at(i));
+
+		if (c.isLetter() && !(c.isUpper() || c == QChar(0x00DF)))
 			return false;
+	}
 
 	return true;
 }
 
 static QString capitalized(const QString &str)
 {
-	QString ret(str);
-	for (int i = 0; i < str.size(); i++)
-		if (i && !str.at(i-1).isSpace())
-			ret[i] = str.at(i).toLower();
-		else
-			ret[i] = str.at(i);
+	QString ret;
+
+	ret.resize(str.size());
+
+	if (!str.isEmpty())
+		ret[0] = str.at(0);
+
+	for (int i = 1; i < str.size(); i++) {
+		QChar last(str.at(i-1));
+		QChar c(str.at(i));
+
+		ret[i] = (last.isSpace() || last == '(') ? c : c.toLower();
+	}
 
 	return ret;
 }
@@ -65,11 +48,10 @@ static QByteArray ft2m(const QByteArray &str)
 	return ok ? QByteArray::number(qRound(number * 0.3048)) : str;
 }
 
+
 LBLFile::~LBLFile()
 {
 	delete _huffmanText;
-	delete[] _table;
-	delete[] _rasters;
 }
 
 bool LBLFile::load(Handle &hdl, const RGNFile *rgn, Handle &rgnHdl)
@@ -93,10 +75,10 @@ bool LBLFile::load(Handle &hdl, const RGNFile *rgn, Handle &rgnHdl)
 			return false;
 
 		if (size && recordSize) {
-			_table = new quint32[size / recordSize];
+			_table.resize(size / recordSize);
 			if (!seek(hdl, offset))
 				return false;
-			for (quint32 i = 0; i < size / recordSize; i++) {
+			for (quint32 i = 0; i < _table.size(); i++) {
 				if (!readVUInt32(hdl, recordSize, _table[i]))
 					return false;
 			}
@@ -129,12 +111,10 @@ bool LBLFile::load(Handle &hdl, const RGNFile *rgn, Handle &rgnHdl)
 
 void LBLFile::clear()
 {
+	_table = QVector<quint32>();
+	_rasters = QVector<Image>();
 	delete _huffmanText;
-	delete[] _table;
-	delete[] _rasters;
 	_huffmanText = 0;
-	_table = 0;
-	_rasters = 0;
 }
 
 Label LBLFile::str2label(const QVector<quint8> &str, bool capitalize,
@@ -183,6 +163,30 @@ Label LBLFile::str2label(const QVector<quint8> &str, bool capitalize,
 Label LBLFile::label6b(const SubFile *file, Handle &fileHdl, quint32 size,
   bool capitalize, bool convert) const
 {
+	static const quint8 NORMAL_CHARS[] = {
+		' ', 'A', 'B', 'C', 'D', 'E', 'F', 'G',
+		'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
+		'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W',
+		'X', 'Y', 'Z',  '~', '~', '~', ' ', ' ',
+		'0', '1', '2', '3', '4', '5', '6', '7',
+		'8', '9', '~', '~', '~', '~', '~', '~'
+	};
+	static const quint8 SYMBOL_CHARS[] = {
+		'@', '!', '"', '#', '$', '%', '&', '\'',
+		'(', ')', '*', '+', ',', '-', '.', '/',
+		'~', '~', '~', '~', '~', '~', '~', '~',
+		'~', '~', ':', ';', '<', '=', '>', '?',
+		'~', '~', '~', '~', '~', '~', '~', '~',
+		'~', '~', '~', '[', '\\', ']', '^', '_'
+	};
+	static const quint8 SPECIAL_CHARS[] = {
+		'`', 'a', 'b', 'c', 'd', 'e', 'f', 'g',
+		'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
+		'p', 'q', 'r', 's', 't', 'u', 'v', 'w',
+		'x', 'y', 'z', '~', '~', '~', '~', '~',
+		'0', '1', '2', '3', '4', '5', '6', '7',
+		'8', '9', '~', '~', '~', '~', '~', '~'
+	};
 	Shield::Type shieldType = Shield::None;
 	QByteArray label, shieldLabel;
 	QByteArray *bap = &label;
@@ -266,42 +270,41 @@ Label LBLFile::label8b(const SubFile *file, Handle &fileHdl, quint32 size,
 Label LBLFile::labelHuffman(Handle &hdl, const SubFile *file, Handle &fileHdl,
   quint32 size, bool capitalize, bool convert)
 {
+	QVector<quint8> tpl;
+
+	if (!_huffmanText->decode(file, fileHdl, size, tpl))
+		return Label();
+	if (!_table.size())
+		return str2label(tpl, capitalize, convert);
+
+
 	QVector<quint8> str;
 
-	if (!_huffmanText->decode(file, fileHdl, size, str))
-		return Label();
-	if (!_table)
-		return str2label(str, capitalize, convert);
+	for (int i = 0; i < tpl.size(); i++) {
+		quint8 c(tpl.at(i));
+		quint32 val = (c < _table.size()) ? _table.at(c) : 0;
 
-
-	QVector<quint8> str2;
-	for (int i = 0; i < str.size(); i++) {
-		quint32 val = _table[str.at(i)];
 		if (val) {
-			quint32 off = _base.offset + ((val & 0x7fffff) << _shift);
-			if (!seek(hdl, off))
+			if (str.size() && str.back() == '\0')
+				str.back() = ' ';
+			else if (str.size())
+				str.append(' ');
+
+			quint32 offset = _base.offset + ((val & 0x7fffff) << _shift);
+			quint32 limit = _base.offset + _base.size - offset;
+			if (!seek(hdl, offset))
 				return Label();
-
-			if (str2.size() && str2.back() == '\0')
-				str2[str2.size() - 1] = ' ';
-			else if (str2.size())
-				str2.append(' ');
-
-			if (!_huffmanText->decode(this, hdl, _base.offset + _base.size - off,
-			  str2))
+			if (!_huffmanText->decode(this, hdl, limit, str))
 				return Label();
 		} else {
-			if (str.at(i) == 7) {
-				str2.append(0);
-				break;
-			}
-			if (str2.size() && str2.back() == '\0')
-				str2[str2.size() - 1] = ' ';
-			str2.append(str.at(i));
+			if (str.size() && str.back() == '\0')
+				str.back() = ' ';
+
+			str.append(c);
 		}
 	}
 
-	return str2label(str2, capitalize, convert);
+	return str2label(str, capitalize, convert);
 }
 
 Label LBLFile::label(Handle &hdl, quint32 offset, bool poi, bool capitalize,
@@ -347,15 +350,15 @@ bool LBLFile::loadRasterTable(Handle &hdl, quint32 offset, quint32 size,
   quint32 recordSize)
 {
 	quint32 prev, cur;
+	quint32 imgCount = size / recordSize;
 
-	_imgCount = size / recordSize;
-	_imgIdSize = byteSize(_imgCount - 1);
-	_rasters = new Image[_imgCount];
+	_imgIdSize = byteSize(imgCount - 1);
+	_rasters.resize(imgCount);
 
 	if (!(seek(hdl, offset) && readVUInt32(hdl, recordSize, prev)))
 		return false;
 
-	for (quint32 i = 1; i < _imgCount; i++) {
+	for (quint32 i = 1; i < imgCount; i++) {
 		if (!readVUInt32(hdl, recordSize, cur))
 			return false;
 
@@ -365,8 +368,8 @@ bool LBLFile::loadRasterTable(Handle &hdl, quint32 offset, quint32 size,
 		prev = cur;
 	}
 
-	_rasters[_imgCount-1].offset = prev;
-	_rasters[_imgCount-1].size = _img.size - prev;
+	_rasters[imgCount-1].offset = prev;
+	_rasters[imgCount-1].size = _img.size - prev;
 
 	return true;
 }
@@ -375,14 +378,14 @@ QPixmap LBLFile::image(Handle &hdl, quint32 id) const
 {
 	QPixmap pm;
 
-	if (id >= _imgCount)
+	if (id >= _rasters.size())
 		return pm;
 
-	if (!seek(hdl, _img.offset + _rasters[id].offset))
+	if (!seek(hdl, _img.offset + _rasters.at(id).offset))
 		return pm;
 	QByteArray ba;
-	ba.resize(_rasters[id].size);
-	if (!read(hdl, ba.data(), _rasters[id].size))
+	ba.resize(_rasters.at(id).size);
+	if (!read(hdl, ba.data(), ba.size()))
 		return pm;
 
 	pm.loadFromData(ba, "jpeg");
